"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleFetchPrograms = exports.saveAccount = exports.getExecutableAddress = void 0;
const web3_js_1 = require("@solana/web3.js");
const child_process_1 = require("child_process");
const utils_1 = require("../utils");
const fs_1 = require("../utils/fs");
const path_1 = __importDefault(require("path"));
const http_1 = require("../utils/http");
async function getExecutableAddress(programId) {
    const programPubkey = new web3_js_1.PublicKey(programId);
    const [executableAddress] = await web3_js_1.PublicKey.findProgramAddress([programPubkey.toBytes()], new web3_js_1.PublicKey('BPFLoaderUpgradeab1e11111111111111111111111'));
    return executableAddress.toString();
}
exports.getExecutableAddress = getExecutableAddress;
async function saveAccount(programId, endpoint, programFolder) {
    const makeEndArgs = (id) => [
        '-u',
        endpoint,
        '-o',
        `${programFolder}/${id}.json`,
        '--output',
        'json',
    ];
    (0, utils_1.logInfo)(`Saving program ${programId} from cluster ${endpoint}`);
    (0, child_process_1.spawnSync)('solana', ['account', programId, ...makeEndArgs(programId)]);
    const executableId = await getExecutableAddress(programId);
    (0, child_process_1.spawnSync)('solana', ['account', executableId, ...makeEndArgs(executableId)]);
}
exports.saveAccount = saveAccount;
async function handleFetchPrograms(programs, programFolder, force = false) {
    (0, fs_1.ensureDirSync)(programFolder);
    if (programs.length > 0) {
        for (const { programId, deployPath } of programs) {
            if ((0, http_1.isValidHttpUrl)(deployPath)) {
                if (force ||
                    !(await (0, fs_1.canAccess)(path_1.default.join(programFolder, `${programId}.json`)))) {
                    try {
                        await saveAccount(programId, deployPath, programFolder);
                    }
                    catch (err) {
                        (0, utils_1.logError)(`Failed to load ${programId} from cluster ${deployPath}`);
                        throw err;
                    }
                }
            }
        }
    }
}
exports.handleFetchPrograms = handleFetchPrograms;
//# sourceMappingURL=local-programs.js.map